@@ -3,12 +3,19 @@
 """
 import base64
 import logging
+import os
+import re
 import time
 from typing import List, Optional, Tuple, Union
 
 import requests
 from requests.models import Response
-from requests_oauthlib import OAuth1, OAuth2, OAuth1Session
+from authlib.integrations.requests_client import (
+    OAuth2Session,
+    OAuth2Auth,
+    OAuth1Auth,
+    OAuth1Session,
+)
 
 import pytwitter.models as md
 from pytwitter.error import PyTwitterError
@@ -24,6 +31,9 @@ class Api:
     BASE_AUTHORIZE_URL = "https://api.twitter.com/oauth/authorize"
     BASE_ACCESS_TOKEN_URL = "https://api.twitter.com/oauth/access_token"
     DEFAULT_CALLBACK_URI = "https://localhost/"
+    BASE_OAUTH2_AUTHORIZE_URL = "https://twitter.com/i/oauth2/authorize"
+    BASE_OAUTH2_ACCESS_TOKEN_URL = "https://api.twitter.com/2/oauth2/token"
+    DEFAULT_SCOPES = ["users.read", "tweet.read"]
 
     def __init__(
         self,
@@ -32,6 +42,7 @@ class Api:
         consumer_secret=None,
         access_token=None,
         access_secret=None,
+        client_id=None,
         application_only_auth=False,
         oauth_flow=False,  # provide access with authorize
         sleep_on_rate_limit=False,
@@ -43,6 +54,7 @@ class Api:
         self._oauth_session = None
         self.consumer_key = consumer_key
         self.consumer_secret = consumer_secret
+        self.client_id = client_id
         self.timeout = timeout
         self.proxies = proxies
         self.rate_limit = RateLimit()
@@ -51,7 +63,7 @@ class Api:
 
         # just use bearer token
         if bearer_token:
-            self._auth = OAuth2(
+            self._auth = OAuth2Auth(
                 token={"access_token": bearer_token, "token_type": "Bearer"}
             )
         # use app auth
@@ -59,16 +71,16 @@ class Api:
             resp = self.generate_bearer_token(
                 consumer_key=consumer_key, consumer_secret=consumer_secret
             )
-            self._auth = OAuth2(
+            self._auth = OAuth2Auth(
                 token={"access_token": resp["access_token"], "token_type": "Bearer"}
             )
         # use user auth
         elif all([consumer_key, consumer_secret, access_token, access_secret]):
-            self._auth = OAuth1(
-                client_key=consumer_key,
+            self._auth = OAuth1Auth(
+                client_id=consumer_key,
                 client_secret=consumer_secret,
-                resource_owner_key=access_token,
-                resource_owner_secret=access_secret,
+                token=access_token,
+                token_secret=access_secret,
             )
             self.rate_limit = RateLimit("user")
             self.auth_user_id = self.get_uid_from_access_token_key(
@@ -77,6 +89,8 @@ class Api:
         # use oauth flow by hand
         elif consumer_key and consumer_secret and oauth_flow:
             pass
+        elif client_id and oauth_flow:
+            pass
         else:
             raise PyTwitterError("Need oauth")
 
@@ -145,16 +159,18 @@ class Api:
         if callback_uri is None:
             callback_uri = self.DEFAULT_CALLBACK_URI
         self._oauth_session = OAuth1Session(
-            client_key=self.consumer_key,
+            client_id=self.consumer_key,
             client_secret=self.consumer_secret,
             callback_uri=callback_uri,
         )
         self._oauth_session.fetch_request_token(
             self.BASE_REQUEST_TOKEN_URL, proxies=self.proxies
         )
-        return self._oauth_session.authorization_url(self.BASE_AUTHORIZE_URL, **kwargs)
+        return self._oauth_session.create_authorization_url(
+            self.BASE_AUTHORIZE_URL, **kwargs
+        )
 
-    def generate_access_token(self, response):
+    def generate_access_token(self, response: str):
         """
         :param response:
         :return:
@@ -167,11 +183,11 @@ class Api:
         data = self._oauth_session.fetch_access_token(
             self.BASE_ACCESS_TOKEN_URL, proxies=self.proxies
         )
-        self._auth = OAuth1(
-            client_key=self.consumer_key,
+        self._auth = OAuth1Auth(
+            client_id=self.consumer_key,
             client_secret=self.consumer_secret,
-            resource_owner_key=data["oauth_token"],
-            resource_owner_secret=data["oauth_token_secret"],
+            token=data["oauth_token"],
+            token_secret=data["oauth_token_secret"],
         )
         if "user_id" in data:
             self.auth_user_id = data["user_id"]
@@ -190,7 +206,7 @@ class Api:
         if not self._auth:
             raise PyTwitterError("Must have authorized credentials")
 
-        if not isinstance(self._auth, OAuth1):
+        if not isinstance(self._auth, OAuth1Auth):
             raise PyTwitterError("Can only revoke oauth1 token")
 
         resp = requests.post(
@@ -242,6 +258,77 @@ class Api:
         data = self._parse_response(resp=resp)
         return data
 
+    def _get_oauth2_session(
+        self,
+        redirect_uri: Optional[str] = None,
+        scope: Optional[List[str]] = None,
+        **kwargs,
+    ) -> OAuth2Session:
+        """
+        :param redirect_uri: The URL that twitter redirect back to after the user logged in.
+        :param scope: A list of permission string to request from the user to using your app.
+        :param kwargs: Additional parameters for oauth.
+        :return: OAuth Session
+        """
+        # check app credentials
+        if not self.client_id:
+            raise PyTwitterError({"message": "OAuth need your app credentials"})
+
+        if redirect_uri is None:
+            redirect_uri = self.DEFAULT_CALLBACK_URI
+        if scope is None:
+            scope = self.DEFAULT_SCOPES
+
+        session = OAuth2Session(
+            client_id=self.client_id,
+            scope=scope,
+            redirect_uri=redirect_uri,
+            code_challenge_method="S256",
+            **kwargs,
+        )
+        return session
+
+    def get_oauth2_authorize_url(
+        self, redirect_uri: str = None, scope: Optional[List[str]] = None, **kwargs
+    ) -> Tuple[str, str, str]:
+        """
+        :param redirect_uri: The URL that twitter redirect back to after the user logged in.
+        :param scope: A list of permission string to request from the user to using your app.
+        :param kwargs: Additional parameters for oauth.
+        :return: Authorization url, code_verifier, state
+        """
+        session = self._get_oauth2_session(
+            redirect_uri=redirect_uri,
+            scope=scope,
+            **kwargs,
+        )
+        code_verifier = base64.urlsafe_b64encode(os.urandom(40)).decode("utf-8")
+        code_verifier = re.sub("[^a-zA-Z0-9]+", "", code_verifier)
+
+        authorization_url, state = session.create_authorization_url(
+            url=self.BASE_OAUTH2_AUTHORIZE_URL, code_verifier=code_verifier
+        )
+        return authorization_url, code_verifier, state
+
+    def generate_oauth2_access_token(
+        self, response: str, code_verifier: str, redirect_uri: str = None
+    ):
+        """
+        :param response: Response url after user logged in.
+        :param code_verifier: Code verifier when your
+        :param redirect_uri:
+        :return:
+        """
+        session = self._get_oauth2_session(redirect_uri=redirect_uri)
+
+        token = session.fetch_token(
+            url=self.BASE_OAUTH2_ACCESS_TOKEN_URL,
+            authorization_response=response,
+            code_verifier=code_verifier,
+        )
+        self._auth = OAuth2Auth(token=token["access_token"])
+        return token
+
     @staticmethod
     def _parse_response(resp: Response) -> dict:
         """
@@ -270,7 +357,7 @@ class Api:
     def _get(
         self,
         url: str,
-        params: dict,
+        params: Optional[dict],
         cls,
         multi: bool = False,
         return_json: bool = False,
@@ -292,7 +379,7 @@ class Api:
             return resp_json
         else:
             data, includes, meta, errors = (
-                resp_json["data"],
+                resp_json.get("data", []),
                 resp_json.get("includes"),
                 resp_json.get("meta"),
                 resp_json.get("errors"),
@@ -1172,6 +1259,50 @@ class Api:
         data = self._parse_response(resp)
         return data
 
+    def get_user_muting(
+        self,
+        user_id: str,
+        *,
+        user_fields: Optional[Union[str, List, Tuple]] = None,
+        expansions: Optional[Union[str, List, Tuple]] = None,
+        tweet_fields: Optional[Union[str, List, Tuple]] = None,
+        max_results: Optional[int] = None,
+        pagination_token: Optional[str] = None,
+        return_json: bool = False,
+    ):
+        """
+        Returns a list of users who are muted by the specified user ID.
+
+        :param user_id: ID for user which you want to get muting.
+        :param user_fields: Fields for the user object.
+        :param expansions: Fields for the expansions now only `pinned_tweet_id`.
+        :param tweet_fields: Fields for the tweet object, Expansions required.
+        :param max_results: The maximum number of results to be returned per page. Number between 1 and the 1000.
+        :param pagination_token: Token for the pagination.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the users.
+            - includes: expansions data.
+            - meta: pagination details
+        """
+
+        args = {
+            "user.fields": enf_comma_separated(name="user_fields", value=user_fields),
+            "expansions": enf_comma_separated(name="expansions", value=expansions),
+            "tweet.fields": enf_comma_separated(
+                name="tweet_fields", value=tweet_fields
+            ),
+            "max_results": max_results,
+            "pagination_token": pagination_token,
+        }
+        return self._get(
+            url=f"{self.BASE_URL_V2}/users/{user_id}/muting",
+            params=args,
+            cls=md.User,
+            multi=True,
+            return_json=return_json,
+        )
+
     def get_tweet_retweeted_users(
         self,
         tweet_id: str,
@@ -1241,3 +1372,260 @@ class Api:
         )
         data = self._parse_response(resp=resp)
         return data
+
+    def get_space(
+        self,
+        space_id: str,
+        *,
+        space_fields: Optional[Union[str, List, Tuple]] = None,
+        expansions: Optional[Union[str, List, Tuple]] = None,
+        user_fields: Optional[Union[str, List, Tuple]] = None,
+        return_json: bool = False,
+    ):
+        """
+        Returns a variety of information about a single Space specified by the requested ID.
+
+        :param space_id: The ID for the target space.
+        :param space_fields: Fields for the space object.
+        :param expansions: Fields for expansions.
+        :param user_fields: Fields for the user object.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the space
+            - includes: expansions data.
+        """
+
+        args = {
+            "space.fields": enf_comma_separated(
+                name="space_fields",
+                value=space_fields,
+            ),
+            "expansions": enf_comma_separated(name="expansions", value=expansions),
+            "user.fields": enf_comma_separated(name="user_fields", value=user_fields),
+        }
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/spaces/{space_id}",
+            params=args,
+            cls=md.Space,
+            return_json=return_json,
+        )
+
+    def get_spaces(
+        self,
+        space_ids: Union[str, List, Tuple],
+        *,
+        space_fields: Optional[Union[str, List, Tuple]] = None,
+        expansions: Optional[Union[str, List, Tuple]] = None,
+        user_fields: Optional[Union[str, List, Tuple]] = None,
+        return_json: bool = False,
+    ):
+        """
+        Returns details for multiple Spaces. Up to 100 comma-separated Spaces IDs can be looked up using this endpoint.
+
+        :param space_ids: The IDs for target spaces, Up to 100 are allowed in a single request.
+        :param space_fields: Fields for the space object.
+        :param expansions: Fields for expansions.
+        :param user_fields: Fields for the user object.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the spaces
+            - includes: expansions data.
+        """
+        args = {
+            "ids": enf_comma_separated(name="space_ids", value=space_ids),
+            "space.fields": enf_comma_separated(
+                name="space_fields",
+                value=space_fields,
+            ),
+            "expansions": enf_comma_separated(name="expansions", value=expansions),
+            "user.fields": enf_comma_separated(name="user_fields", value=user_fields),
+        }
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/spaces",
+            params=args,
+            cls=md.Space,
+            multi=True,
+            return_json=return_json,
+        )
+
+    def get_spaces_by_creator(
+        self,
+        creator_ids: Union[str, List, Tuple],
+        *,
+        space_fields: Optional[Union[str, List, Tuple]] = None,
+        expansions: Optional[Union[str, List, Tuple]] = None,
+        user_fields: Optional[Union[str, List, Tuple]] = None,
+        max_results: Optional[int] = None,
+        return_json: bool = False,
+    ):
+        """
+        Returns live or scheduled Spaces created by the specified user IDs.
+        Up to 100 comma-separated IDs can be looked up using this endpoint.
+
+        :param creator_ids: IDs for the creators, Up to 100 are allowed in a single request.
+        :param space_fields: Fields for the space object.
+        :param expansions: Fields for expansions.
+        :param user_fields: Fields for the user object.
+        :param max_results: The maximum number of results to be returned per page. Number between 1 and the 100.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the spaces
+            - includes: expansions data.
+        """
+        args = {
+            "user_ids": enf_comma_separated(name="creator_ids", value=creator_ids),
+            "space.fields": enf_comma_separated(
+                name="space_fields",
+                value=space_fields,
+            ),
+            "expansions": enf_comma_separated(name="expansions", value=expansions),
+            "user.fields": enf_comma_separated(name="user_fields", value=user_fields),
+            "max_results": max_results,
+        }
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/spaces/by/creator_ids",
+            params=args,
+            cls=md.Space,
+            multi=True,
+            return_json=return_json,
+        )
+
+    def search_spaces(
+        self,
+        query: str,
+        state: str,
+        *,
+        space_fields: Optional[Union[str, List, Tuple]] = None,
+        expansions: Optional[Union[str, List, Tuple]] = None,
+        user_fields: Optional[Union[str, List, Tuple]] = None,
+        max_results: Optional[int] = None,
+        return_json: bool = False,
+    ):
+        """
+        Return live or scheduled Spaces matching your specified search terms
+
+        :param query: Your search term.
+            This can be any text (including mentions and Hashtags) present in the title of the Space.
+        :param state: Determines the type of results to return.
+            Accepted values: live,scheduled
+        :param space_fields: Fields for the space object.
+        :param expansions: Fields for expansions.
+        :param user_fields: Fields for the user object.
+        :param max_results: The maximum number of results to be returned per page. Number between 1 and the 100.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the spaces
+            - includes: expansions data.
+        """
+
+        args = {
+            "query": query,
+            "state": state,
+            "space.fields": enf_comma_separated(
+                name="space_fields",
+                value=space_fields,
+            ),
+            "expansions": enf_comma_separated(name="expansions", value=expansions),
+            "user.fields": enf_comma_separated(name="user_fields", value=user_fields),
+            "max_results": max_results,
+        }
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/spaces/search",
+            params=args,
+            cls=md.Space,
+            multi=True,
+            return_json=return_json,
+        )
+
+    def get_compliance_job(
+        self,
+        job_id: str,
+        *,
+        return_json: bool = False,
+    ):
+        """
+        Get a single compliance job with the specified ID.
+
+        :param job_id: ID for the compliance job.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the job.
+        """
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/compliance/jobs/{job_id}",
+            params=None,
+            cls=md.ComplianceJob,
+            return_json=return_json,
+        )
+
+    def get_compliance_jobs(
+        self,
+        job_type: Optional[str],
+        *,
+        status: Optional[str] = None,
+        return_json: bool = False,
+    ):
+        """
+        Returns a list of recent compliance jobs.
+
+        :param job_type: Type for the job, Accepted values are: tweets, users.
+        :param status: Status for the job.
+            Accepted values are: created, in_progress, failed, complete.
+            Default is all.
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return:
+            - data: data for the jobs.
+        """
+
+        args = {"type": job_type}
+        if status:
+            args["status"] = status
+
+        return self._get(
+            url=f"{self.BASE_URL_V2}/compliance/jobs",
+            params=args,
+            cls=md.ComplianceJob,
+            multi=True,
+            return_json=return_json,
+        )
+
+    def create_compliance_job(
+        self,
+        job_type: Optional[str],
+        *,
+        name: Optional[str] = None,
+        resumable: Optional[bool] = None,
+        return_json: bool = False,
+    ):
+        """
+        Creates a new compliance job for Tweet IDs or user IDs.
+        You can run one batch job at a time.
+
+        :param job_type: Type for the job, Accepted values are: tweets, users.
+        :param name: A name for this job, useful to identify multiple jobs using a label you define.
+        :param resumable: Specifies whether to enable the upload URL with support for resumable uploads
+        :param return_json: Type for returned data. If you set True JSON data will be returned.
+        :return: Compliance job information.
+        """
+
+        args = {"type": job_type}
+        if name is not None:
+            args["name"] = name
+        if resumable is not None:
+            args["resumable"] = resumable
+
+        resp = self._request(
+            url=f"{self.BASE_URL_V2}/compliance/jobs",
+            verb="POST",
+            json=args,
+        )
+        data = self._parse_response(resp=resp)
+        if return_json:
+            return data
+        else:
+            return md.ComplianceJob.new_from_json_dict(data["data"])
