@@ -3,5 +3,7 @@ from .tweet import *  # noqa
 from .media import *  # noqa
 from .poll import *  # noqa
 from .place import *  # noqa
+from .compliance import *  # noqa
+from .space import *  # noqa
 from .stream import *  # noqa
 from .ext import *  # noqa
